@@ -74,9 +74,12 @@ c############################
       real*8  v, vz, elran, vt
 c########################
 
+      real amsave(15)
+
       data cdddd/0./
       data ceet/0.5/
 
+      
 
       data iamc/1,2,8,9,12,15,16,17,18,19,20,21,22,24,25/
       data ins/13,13,5*32,14,14,3*32,10,10
@@ -90,8 +93,10 @@ c########################
 
       data ixpi/1,-1,5*2,1,-1,3*2,1,0,1,0,4,4,2,3*4,1,1,1,5*2/
       data ieveno/0/
-      save ieveno
-      
+      save ieveno, amsave
+
+
+
       itnucr=itttt
       if (itprf( itttt).lt.0) go to 99999
       write (6,99998)itttt
@@ -102,13 +107,19 @@ c########################
       sico=1.
       elph=0.
       plabco=1.
-
-      if(ieveno .ge. 1 ) then
 c  &&&&&&&&&&&&&6666666 since am is changed inside
 c           and might not be restored, we every time
-c           restore from the second call  KK v.6.14
-         do iam=1,15
-            am(iamc(iam)) = amhh(iam)
+c           restore it from the second call  KK v.6.16
+c
+      if(ieveno .eq. 0 ) then
+c              save mass
+         do i = 1, 15
+            amsave(i)=  am(iamc(i))
+         enddo
+      else
+c               restore  
+         do i=1,15
+            am(iamc(i)) = amsave(i)
          enddo
       endif
 c    &&&&&&&&&&&&&&
@@ -539,8 +550,6 @@ c*** 1.88gev-threshold cm-energy in annihilation)
 c--------------------------------------------------
       do 1511 iam=1,15
       jam=iamc(iam)
-c   &&&&&&&&&&&&&&&6 this is questionable.  am(24) kaon mass
-c                    is changed and lead to error
 1511  am(jam)=t3
 
 1510  continue
@@ -641,6 +650,7 @@ c*** if no anihilation into mesons for an+n
 c--------------------------------------------------
       if (ir.lt.2.and.inxpi.lt.0) go to 510
 88    continue
+
       do 1512 iam=1,15
 c
 c--------------------------------------------------
