@@ -3,6 +3,7 @@ c
       subroutine cbeginRun
       implicit none
 #include  "Zmanagerp.h"
+#include  "Zmanager.h"
 #include  "Zelemagp.h"
 #include  "Zevhnp.h"
 #include  "Ztrack.h"
@@ -15,19 +16,21 @@ c
 #include  "Zobs.h"
 #include  "Zobsp.h"
 
-      character*70 msg
+
       character*16 temp
       integer jold, icon
       real*8 s1
 
+
       if(Cont) then
 c           restore status at the end of previous run
          call crestoreStatus
-         Cont = .true.
-      elseif(Job .eq. 'flesh' ) then
+         Cont =.true.
+      elseif(Job .eq. 'flesh'  .or. Job .eq. 'newflesh') then
+         temp = Job
          call cgetSkelFile 
-         Job = 'flesh'    ! Job='flesh' is kept as now
-      elseif(Job .eq. 'skeleton') then
+         Job = temp    !  keet as now
+      elseif(Job .eq. 'skeleton' .or. Job .eq. 'newskel' ) then
 c            to be safe, parameters are written in a file first and read
 c            again ; this will avoid the possible difference of internal
 c            parameter values read from original param and SkeletonParam
@@ -37,16 +40,25 @@ c
             call cerrorMsg(SkeletonFile, 1)
             call cerrorMsg('File shown above cannot be opened',0)
          endif
+c           at flesh time, you don't need to rewrite Job
+         temp = Job
+         if(Job .eq. 'newskel') then
+            Job = 'newflesh'
+         else
+            Job = 'flesh'
+         endif
          call cwriteParam(TempDev, 1)
          close(TempDev)
          call copenf(TempDev, SkeletonFile, icon)
          call creadParam(TempDev)
          close(TempDev)
+c            restore Job
+         Job = temp
       endif
 
 c        reset parameters.
       if(BorderHeightH .eq. 0.0) then
-         BorderHeightH = HeightOfInj
+         BorderHeightH = HeightOfInj + 1.0d0    ! from v6.10
       endif
 
       if(DestEventNo(2) .eq. 0) then
@@ -63,18 +75,7 @@ c         to lower case letters.
 
       call rndsw(jold, 1)    ! specify random number generator 1.
 
-      if(IntModel .eq. 'int1') then
-c           for Lund init.  
-         call haddenC
-         call chanwnC
-      elseif(IntModel .eq. 'int2') then
-c         Gheisha; init. Gheisha cannot use hadrin/nucrin in Cosmos
-         call gpart
-         call gheini
-      else
-         write(msg, *) 'IntModel=', IntModel,' not supported'
-         call cerrorMsg(msg, 0)
-      endif
+      call cintModels('cosmos')  ! analysis of interaction models and init.
 #if ATMOSPHERE == 1
 c          read segmented atmosphere data
       call creadAtmosD
@@ -101,26 +102,34 @@ c       init for muon interaction routines; specific for Air.
       FromEpics = .false.  ! muon interaction routines for Air are
                            ! inside Cosmos 
       call ciniSPrimAng    !  this is in csPrimaAgn.f in Tracking dir.
-      call ciniTracking0  !  init for  tracking for all events
-
 c        check job
       call cwhatJob
-
+c          this is moved here; before v6.10 it was before cwhatJob
+      call ciniTracking0  !  init for  tracking for all events
 c           init for knockon process; Knockon is not used now.
       if(KnockOnRatio .lt. 1.d0) then
+c              ********* Knockon is not used now ******
+         if(Job .eq. 'newskel') then
+            call cdedxEleci(KEminObs*KnockOnRatio, Knockon)
+         else
 c                 this must come after cwhatJob, since KEminObs2 must
 c           be fixed.  For skeleton-flesh job, KnockOnRatio should be
 c           small enough so that KEminObs2*KnockOnRatio < KEminObs at
 c           flesh time  
-         call cdedxEleci(KEminObs2*KnockOnRatio, Knockon)
+
+            call cdedxEleci(KEminObs2*KnockOnRatio, Knockon)
+         endif
       else
          call cdedxEleci(RecoilKineMinE, Knockon)
       endif
 c         user hook
       call chookBgRun
+      
       end
+
       subroutine ciniTracking0
       implicit none
+#include  "Zmanager.h"
 #include  "Zmanagerp.h"
 #include  "Zevhnp.h"
 #include  "Ztrack.h"
@@ -134,8 +143,18 @@ c         user hook
 #include  "Zheavyv.h"
 
 c
-      integer i
+      integer i, now(2)
+c  
+c      ********** 
+c      next  is a "directive" for absoft f90.
+c      which is to make the call TIME as if call time.
+c      For absoft f90 ; we have to treat all variable in upper case
+c      but there is no TIME tourine in the system.
+c
+cdir$ NAME (TIME="time")
 
+c
+c
       ObserveAS = index(Generate, 'as')  .gt. 0 .or.
      *            index(Generate, 'lat') .gt. 0 
       if(index(Generate, 'qas') .gt. 0) then
@@ -145,8 +164,19 @@ c
       endif
 
       EventsInTheRun = 0
-      if(InitRN(1) .ne. 0) then
+
+      RefreshIR = InitRN(1) .lt. 0 .and. 
+     *        ( Job .ne. 'flesh' .and. Job .ne. 'newflesh')
+
+      if(InitRN(1) .gt. 0 .and. InitRN(2) .gt. 0 ) then
          call rnd1r(InitRN)     ! init randeom number generator
+c      *****************
+      elseif(.not. RefreshIR .and. InitRN(2) .lt. 0) then
+
+         call time(now(1))
+         now(2) =-InitRN(2)
+         call rnd1r(now)
+c     ******************
       endif
 c          compute the offset point in 'xyz' system
 c        the deepest detector origin + Offset is the point
@@ -181,13 +211,27 @@ c
       character*190  msg
       integer klena, icon
       character*8  uid 
+      character*16 temp
 
-      if(Job .eq. ' ' .or. Job .eq. 'skeleton') then
-c          save present conditions
-         KEminObs2 = KEminObs
-         Generate2 = Generate
-         NoOfSites2 = NoOfSites
-         EndLevel2 = EndLevel
+      if(Job .eq. ' ' .or. Job .eq. 'skeleton' .or. 
+     *   Job .eq. 'newskel'  ) then
+         if(Job .ne. 'newskel') then
+c             save present conditions
+            KEminObs2 = KEminObs
+            Generate2 = Generate
+            EndLevel2 = EndLevel
+         elseif(Job .eq. 'newskel') then
+            if( KEminObs2 .ge. KEminObs .and.
+     *          EndLevel2 .le. EndLevel .and.
+     *          index(Generate2,'as') .eq. 0 .and.
+     *          index(Generate2,'lat') .eq. 0 ) then
+               call cerrorMsg(
+     *          'Doing newskel job seems nonsense', 1)
+               call cerrorMsg(
+     *          'Check Generate2, KEminObs2, EndLevel2',0)
+            endif
+         endif
+         NoOfSites2 = NoOfSites    ! probably not needed
          if(Job .eq. ' ') then
             if(SeedFile .ne. ' ') then
 c                  open seed file for output
@@ -203,7 +247,7 @@ c                  open seed file for output
                   call cskiptoEOF(SeedFileDev)
                endif
             endif               
-         elseif(Job .eq. 'skeleton') then
+         elseif(Job .eq. 'skeleton' .or. Job .eq. 'newskel' ) then
             call
      *      cerrorMsg('  ********** skeleton making **********', 1)
             write(msg, *)  '      Generate=', Generate
@@ -212,13 +256,23 @@ c                 save skeleton inf. in skelotonFile file.
 c             The file will be modified when the distjob command
 c             processes Job = 'flesh' later. You need not modify
 c             skeleton file if distjob is employed.
-            call copenfw(TempDev, SkeletonFile, icon)
-            if(icon .ne. 0) then
-               call cerrorMsg(SkeletonFile, 1)
-               call cerrorMsg('File shown above cannot be opened',0)
+            if(.not. Cont) then
+               temp = Job    ! save current Job
+               if(Job .eq. 'newskel') then
+                  Job = 'newflesh'
+               else
+                  Job = 'flesh'
+               endif
+               call copenfw(TempDev, SkeletonFile, icon)
+               if(icon .ne. 0) then
+                  call cerrorMsg(SkeletonFile, 1)
+                  call cerrorMsg(
+     *              'File shown above cannot be opened',0)
+               endif
+               call cwriteParam(TempDev, 1)
+               close(TempDev)
+               Job = temp
             endif
-            call cwriteParam(TempDev, 1)
-            close(TempDev)
 c                open SeedFile 
             if(SeedFile .eq. ' ') then
 c                error. you need file
@@ -239,25 +293,41 @@ c                error. you need file
                endif
             endif
          endif
-      elseif(Job   .eq. 'flesh') then
+      elseif(Job   .eq. 'flesh' .or. Job .eq. 'newflesh') then
 c             don't worry about KEminObs2 etc.  They have been read
 c             from &Hparam
           call cerrorMsg('  ********** fleshing job   *********', 1)
-          write(msg, *)  '      Old Generate=', Generate2
-          call cerrorMsg(msg, 1)
-          write(msg, *)  '      New Generate=', Generate
-          call cerrorMsg(msg, 1)
-          if(EndLevel .gt. EndLevel2) then
+          if(Job .eq. 'flesh') then
+             if(EndLevel .gt. EndLevel2) then
 c                to deeper detph than skeleton
-             write(msg, *)
-     *        ' fleshing will be done to deeper depth than'//
-     *        ' skeleton making time'
+                write(msg, *)
+     *          ' fleshing will be done to deeper depth than'//
+     *               ' skeleton making time'
+                call cerrorMsg(msg, 1)
+                write(msg, *) ' No of old levels=', EndLevel2,
+     *               ' No of new levels=', EndLevel
+                call cerrorMsg(msg, 1)
+             elseif(EndLevel .lt. EndLevel2) then
+                call cerrorMsg('EndLevel must be >= skelton time', 0)
+             endif
+             write(msg, *)  '      Old Generate=', Generate2
              call cerrorMsg(msg, 1)
-             write(msg, *) ' No of old levels=', EndLevel2,
-     *       ' No of new levels=', EndLevel
+             write(msg, *)  '      New Generate=', Generate
              call cerrorMsg(msg, 1)
-          elseif(EndLevel .lt. EndLevel2) then
-             call cerrorMsg('EndLevel must be >= skelton time', 0)
+          else
+             if(EndLevel .lt. EndLevel2) then
+c                to deeper detph than skeleton
+                write(msg, *)
+     *          ' fleshing will be done to deeper depth than'//
+     *           ' skeleton making time'
+                call cerrorMsg(msg, 1)
+             endif
+c
+c              copy  old Generate, KEminObs2 to current value
+c              they are future values at newskel time
+             KEminObs = KEminObs2
+             Generate = Generate2
+             EndLevel = EndLevel2
           endif
 c                open SeedFile
           if(SeedFile .eq. ' ') then
