@@ -25,21 +25,8 @@ c           get one particle from stack
          if(moreptcl .eq. 0) goto 100   !  exit
          call rndsw(jold, 1)            !  specify 1st generator.
          call cifDead(TrackBefMove, icon)  ! generator may be switched to 2
-c            AS generation. 
 
-c         if(icon .eq. 0) then
-c            thin sampling
-c            if(TrackBefMove.p.fm.p(4) .lt. Ethin) then
-c                do thin sampling
-c               iwgt = TrackBefMove.p.fm.p(4)/Ethin ! inverse weight
-c               call rndc(u)
-c               if(u .gt. iwgt) then
-c                  icon =2       !  discard this ptcl
-c               else
-c                  TrackBefMove.wgt = TrackBefMove.wgt /iwgt
-c               endif
-c            endif    
-c         endif
+c            AS generation. 
 
          if(ObserveAS .and. icon .ne. 2) then
             if(TrackBefMove.p.code .eq. kelec) then
@@ -64,6 +51,16 @@ c         endif
 c        ----------------------------
          if(icon .eq. 0) then
             call cTracking
+         else
+            MoveStat = Dead
+#if DETAILED_TRACKING >=1
+            MovedTrack = TrackBefMove
+            if(icon .eq. 1) then
+               call cobservation(5)
+            else
+               call cobservation(6)
+            endif
+#endif
          endif
       enddo
  100  continue
@@ -80,168 +77,261 @@ c -------------------------------------tracking a ptcl
 #include  "Zobsv.h"
 #include  "Zincidentv.h"
 #include  "Zcode.h"
-
+#include  "Zmanagerp.h"
 c                next is for DEBUG
+c//////
+c      logical deb
+c      common /cdebug/ deb
+c/////////
+ccc c#define DEBUG 1
+
 
 
       real*8 cosangle
       character*70 msg
+      integer nextwhere, never
 cc      integer icon
 
+#if DEBUG == 1
+      call checkstat('before fixModel')
+#endif
 
-         call cfixMedia   ! 
+      call cfixModel( TrackBefMove.p )      ! fix interaction model.
+      call cfixMedia            ! 
 c           sample interaction length.  The path may
 c           be truncated to a shorter one than really sampled.
 c           In that case, MoveStat == Truncated.
-         call csampIntL
 #if DEBUG == 1
-         call checkstat('after csampIntl')
+      call checkstat('before csampIntl')
+#endif
+
+      call csampIntL
+
+#if DEBUG == 1
+      call checkstat('after csampIntl')
 #endif
 c           Truncate the path if it is too long. Note, the truncated
 c           path in the above process may be again truncated to
 c           a shorter one. This happens, for example, if ptcl energy
 c           is low.  Path inf is reset in IntInfArray.
-         call ctruncPath
-
-
+      call ctruncPath
 #if DEBUG == 1
-         call checkstat('after ctruncPath')
+      call checkstat('after ctruncPath')
 #endif
 
 c           compute path end inf. including scattering and
 c           mag. deflection.  Path end inf is set in MovedTrack
-         call ccompPathEnd
-
+      call ccompPathEnd
 #if DEBUG == 1
-         call checkstat('after ccompPathEnd')
+      call checkstat('after ccompPathEnd')
 #endif
 c           see if MovedTrack crosses an observation site.
 c         (or reaches BorderHeightL)  If so, reset MovedTrack. 
-            call cifXObsSite
-
-#if DEBUG == 4
-            call checkstat('after cifXObsSite')
+      call cifXObsSite(nextwhere)
+#if DEBUG == 1
+      call checkstat('after cifXObsSite')
+#endif
+      if(Trace .ne. 0) then
+         call cputTrInfo        ! put trace info.
+      endif
+#if DETAILED_TRACKING >= 3
+c            the user may kill the ptcl
+      call cobservation(8)
 #endif
 
-         if(Trace .ne. 0) then
-            call cputTrInfo     ! put trace info.
-         endif
+
 c     
-         if(MoveStat .eq. Truncated) then
+      if(MoveStat .eq. Truncated) then
 c           see if truncation is really ok.  for energy = 0, 
 c           if anti ptcl, it should be anihilated, if decayabl,
 c           it should be decayed.
-            if(MovedTrack.p.fm.p(4) .eq. MovedTrack.p.mass) then
-               if(MovedTrack.p.subcode .eq. antip) then
-                  MoveStat = ToInteract
-               elseif(MovedTrack.p.code .eq. kmuon) then
-                  MoveStat = ToInteract
-               elseif(MovedTrack.p.code .eq. kpion) then
-                  MoveStat = ToInteract
-               elseif(MovedTrack.p.code .eq. kkaon) then
-                  MoveStat = ToInteract
-               endif
-            endif
-         endif 
-         if(MoveStat .eq. Truncated) then
-c              stack current ptcl
-              call cpush2(MovedTrack)
-         elseif(MoveStat .eq. ToInteract ) then
-            if(Zfirst .eq. 0.) then
-               Zfirst = MovedTrack.pos.depth
-c                  reset minimum time to reach the obs level (time from this
-c                  hight to the obs. level), if no mag. exists until
-c                  the first collision point. or ptcl goes streight
-               if(mod(HowGeomag, 2) .eq. 1 .or. 
-     *             IncidentCopy.p.charge .eq. 0) then
-                  call csetMinTime(MovedTrack)
-               endif
-            endif
-            call cinteraction
-         elseif(MoveStat .eq. ToBeObserved) then
-            if(MovedTrack.p.fm.p(4)-MovedTrack.p.mass 
-     *          .gt. KEminObs) then
-c                  call only for high energy particles
-c                  (at AS generation, some ptcl may have < KEminObs)
-               call cobservation(2)
-            endif
-c               update observation place
-            if(abs(ObsPlane) .eq. horizontal) then
-               cosangle = MovedTrack.vec.coszenith
-            elseif(abs(ObsPlane) .eq. perpendicular) then
-               call cscalerProd(MovedTrack.vec.w, DcAtObsXyz,
-     *               cosangle)
-            elseif(ObsPlane .eq. spherical) then
-               call cscalerProd(MovedTrack.vec.w, MovedTrack.pos,
-     *               cosangle)
-               cosangle = - cosangle
-            else
-               call 
-     *         cerrorMsg('should not come here in tracking', 0)
+         if(MovedTrack.p.fm.p(4) .eq. MovedTrack.p.mass) then
+            if(MovedTrack.p.subcode .eq. antip) then
+               MoveStat = ToInteract
+            elseif(MovedTrack.p.code .eq. kmuon) then
+               MoveStat = ToInteract
+            elseif(MovedTrack.p.code .eq. kpion) then
+               MoveStat = ToInteract
+            elseif(MovedTrack.p.code .eq. kkaon) then
+               MoveStat = ToInteract
             endif
+         endif
+      endif 
 
+      if(MoveStat .eq. Truncated) then
+c        &&&&&&&&&&&&&& some may lose energy by dE/dx etc. 
+c                       This should be recorded
+         if(Job .eq. 'newskel' .and. 
+     *        (MovedTrack.p.fm.p(4) - MovedTrack.p.mass) .lt. 
+     *        KEminObs)  then
+            never = -1
+            call chookEInt(never)
+         endif
+c        &&&&&&&&&&&&
+c              stack current ptcl
+         call cpush(MovedTrack)
+      elseif(MoveStat .eq. ToInteract ) then
+         if(Zfirst.pos.depth .eq. 0.) then
+c             reset minimum time to reach the obs level (time from this
+c             hight to the obs. level), if no mag. exists until
+c                  the first collision point. or ptcl goes streight
+            call cresetTimer(MovedTrack)
+            Zfirst = MovedTrack
+         endif
+#if DETAILED_TRACKING >= 1 
+         call cobservation(4)
+#endif
+
+#if DEBUG == 1
+         call checkstat('before cinteraction')
+#endif
+         call cinteraction
+c/////////////
+c         if(deb) then
+c            write(*,*) 'after cinte'
+c         endif
+c////////////
+      elseif(MoveStat .eq. ToBeObserved ) then
+         if(MovedTrack.p.fm.p(4)-MovedTrack.p.mass 
+     *        .ge. KEminObs) then    ! .gt. for <= uv6.00
+c             call only for high energy particles
+c                  (at AS generation, some ptcl may have < KEminObs
+c                  or some may loose energy by dE/dx etc.
+
+            call cobservation(2)
+
+            if(Job .eq. 'newskel' .and.
+     *          EndLevel .lt. NoOfSites .and. 
+     *          MovedTrack.where .eq. EndLevel ) then
+c                in this case, even E> KEminObs must be recorded
+c                 for skeleton making for kahanshin
+c                  where should be +1 at flesh time
+               MovedTrack.where = MovedTrack.where + 1
+               never = -3
+               call chookEInt(never)
+               MovedTrack.where = MovedTrack.where - 1 ! for safty
+            endif
+         elseif(Job .eq. 'newskel'  .and. 
+     *          EndLevel .eq. NoOfSites .and.
+     *          MovedTrack.where .lt. EndLevel )  then
+c            &&&&&&  this is to be recorded
+c                where should be + 1 at flesh time
+            MovedTrack.where = MovedTrack.where + 1
+            never = -2
+            call chookEInt(never)
+            MovedTrack.where = MovedTrack.where - 1
+c            &&&&& 
+         endif
+c               update observation place
+         if(abs(ObsPlane) .eq. perpendicular) then
+            call cscalerProd(MovedTrack.vec.w, DcAtObsXyz,
+     *               cosangle)
             if(cosangle .lt. 0.) then
                MovedTrack.where = MovedTrack.where - 1
             else
                MovedTrack.where = MovedTrack.where + 1
             endif
 c                  if more obs-site, stack current ptcl.
-            if(.not. Upgoing) then  ! incident is downgoing
+            if(.not. Upgoing) then ! incident is downgoing
                if(MovedTrack.where .gt. EndLevel) then
-                  ! no need to stack; discard ptcl.
+                                ! no need to stack; discard ptcl.
                else
-                  MovedTrack.where =max( MovedTrack.where, 1)
-                  call cpush2(MovedTrack)
+                  MovedTrack.where =max( MovedTrack.where*1, 1)
+                  call cpush(MovedTrack)
                endif
-            else    !incident is  Upgoing
+            else                !incident is  Upgoing
                if(MovedTrack.where .lt.1 ) then
-                   ! no need to stack
+                                ! no need to stack
                else
                   MovedTrack.where =
-     *                         min(MovedTrack.where, EndLevel)
-                  call cpush2(MovedTrack)
+     *                         min(MovedTrack.where*1, EndLevel)
+                  call cpush(MovedTrack)
                endif
             endif
-         elseif(MoveStat .eq. BorderL) then
-            call cobservation(3)
-         elseif(MoveStat .eq. BorderH) then
-            call cobservation(1)
-         elseif(MoveStat .eq. Dead) then   
-c              nothing to do.
          else
-            write(msg, *) ' movestat=',MoveStat,' invalid'
-           call cerrorMsg(msg, 0)
-         endif   
-
+            MovedTrack.where = nextwhere
+            call cpush(MovedTrack)
+         endif
+      elseif(MoveStat .eq. BorderL) then
+         call cobservation(3)
+      elseif(MoveStat .eq. BorderH) then
+         call cobservation(1)
+      elseif(MoveStat .eq. Dead) then   
+c            actually this will not happen. branch is
+c           made before this
+#if DETAILED_TRACKING >= 1
+         call cobservation(5)
+#endif
+      elseif(MoveStat .eq. AngleLimit) then
+#if DETAILED_TRACKING >= 2
+         call cobservation(7)
+#endif
+      else
+         write(msg, *) ' movestat=',MoveStat,' invalid'
+         call cerrorMsg(msg, 0)
+      endif   
       end
+      subroutine cresetTimer(aTrack)
+      implicit none
+#include  "Ztrack.h"
+#include  "Ztrackv.h"
+#include  "Ztrackp.h"
+#include  "Zobs.h"
+#include  "Zobsp.h"
+#include  "Zobsv.h"
+#include  "Zincidentv.h"
+#include  "Zcode.h"
+#include  "Zmanagerp.h"
+      record /track/ aTrack
+c             reset minimum time to reach the obs level (time from this
+c             hight to the obs. level), if no mag. exists until
+c                  the first collision point. or ptcl goes streight
+      if(mod(HowGeomag, 2) .eq. 1 .or. 
+     *     IncidentCopy.p.charge .eq. 0 .and.
+     *     ObsPlane .ne. 3) then
+         call csetMinTime(aTrack)
+      else
+c         MovedTrack.t = 0.     ! bug bef uv6.09
+      endif
+      end
+
 #if DEBUG > 0
       subroutine checkstat(str)
       implicit none
-
 #include  "Ztrack.h"
 #include  "Ztrackv.h"
 #include  "Ztrackp.h"
 #include  "Zobs.h"
 #include  "Zobsv.h"
+
+      logical deb
+      common /cdebug/deb
+
       character*(*) str
       integer i
+      integer nc
+      data nc/0/
+      nc = nc + 1
+c      if(nc .gt. 750) then
+      if( deb) then
+
 
-c         if(TrackBefMove.p.code .eq. 6
-c     *     .and. TrackBefMove.p.charge .eq. -1) then
             write(*,*) str, ' code=',TrackBefMove.p.code,
      *         ' cg',TrackBefMove.p.charge,
      *         ' e=', TrackBefMove.p.fm.p(4), ' stat=',MoveStat
             write(*, *) ' where', TrackBefMove.where
             write(*,*)  ' h==', TrackBefMove.pos.height,
      *       ' d=', TrackBefMove.pos.depth
-c            write(*,*) ' cos=',TrackBefMove.vec.coszenith
-c            do i = 1, NumberOfInte
-c               write(*, *) ' process=',IntInfArray(i).process
-c               write(*, *) ' thickness=',IntInfArray(i).thickness
-c               write(*, *) ' length=',IntInfArray(1).length
-c            enddo
+            write(*,*) ' cos=',TrackBefMove.vec.coszenith
+            do i = 1, NumberOfInte
+               write(*, *) ' process=',IntInfArray(i).process
+               write(*, *) ' thickness=',IntInfArray(i).thickness
+               write(*, *) ' length=',IntInfArray(1).length
+            enddo
+            write(*,*)'  ProcessNo=', ProcessNo
             write(*,*)  '--------------'
 c         endif
+         endif
       end
 #endif
 c-------------------------------------------------------
@@ -284,15 +374,19 @@ c         argon
       if(HowGeomag .le. 2 .or. HowGeomag .eq. 31) then
 c             distant; change of B is < 1 % 
          call clengSmallBC(TrackBefMove, distant)
-
+c   **********
+         distant = distant/10.
+c   **********
          if( (TrackBefMove.pos.xyz.r(1) - x)**2 + 
      *        (TrackBefMove.pos.xyz.r(2) - y)**2 + 
      *        (TrackBefMove.pos.xyz.r(3) - z)**2 
      *        .gt. distant**2) then
 c              if more than MagChgDist m from previous mag set.,
 c               reset mag field.
+c
             call cgeomag(YearOfGeomag,  TrackBefMove.pos.xyz, 
      *                Mag, icon)
+
             call ctransMagTo('xyz', TrackBefMove.pos.xyz, 
      *        Mag, Mag)
             x = TrackBefMove.pos.xyz.r(1)
@@ -355,7 +449,7 @@ c
             call csampNEPIntL   ! non Electron Photon  Interaction.
          endif
          call cfixProc
-         if(.not. Freec .and. Zfirst .eq. 0.) then
+         if(.not. Freec .and. Zfirst.pos.depth .eq. 0.) then
             IntInfArray(ProcessNo).length = 0.
             IntInfArray(ProcessNo).thickness = 0.
          endif
@@ -371,7 +465,7 @@ c     **************************** cfixProc **********
 #include  "Zearth.h"
 
 
-      real*8 h, leng, t, minlen, clen2thick
+      real*8 h, leng, t, minlen, clen2thick, pcut
 c     
       integer i, jcut
 c     
@@ -384,8 +478,10 @@ c              get vertical height
 c              convert kg/m2 into length
             if(NumberOfInte .eq. 1 .or.
      *        IntInfArray(i).thickness .ne. Infty) then
-               call cthick2len(h, TrackBefMove.vec.coszenith, 
-     *         IntInfArray(i).thickness, leng, t, jcut)
+c               call cthick2len(h, TrackBefMove.vec.coszenith, 
+c     *         IntInfArray(i).thickness, leng, t, jcut)
+               call cthick2len(TrackBefMove,
+     *         IntInfArray(i).thickness, leng, t, jcut) 
                if(leng .le. 0.) then
                   leng = 1.d-3
                endif
@@ -408,7 +504,27 @@ c              convert kg/m2 into length
             minlen = leng
          endif
       enddo
+c      if(leng .ge. minlen) then  !   bug in 6.00
+      if(leng .ge. Infty) then  
+         ProcessNo = 1
+         IntInfArray(1).length = 1.e5
+      endif         
       if(IntInfArray(ProcessNo).decay .or. jcut .ne. 0) then
+c         In the case  of muon, if individual knockon process  
+c       is neglected (by parameter setting or with high Emin)
+c       the length could be quite large (say, 6000  km).
+c       and result in the  error in the next call.
+c       To avoid that, we cut the path here
+c
+         if( TrackBefMove.vec.coszenith .lt. 0.) then
+            pcut = 300.d3
+         else
+            pcut = 30.d3
+         endif
+         if(IntInfArray(ProcessNo).length .gt. pcut) then
+            MoveStat = Truncated
+            IntInfArray(ProcessNo).length  = pcut
+         endif
          IntInfArray(ProcessNo).thickness = clen2thick(h, 
      *      TrackBefMove.vec.coszenith, 
      *      IntInfArray(ProcessNo).length )
@@ -428,7 +544,8 @@ c        an observation depth.
 c
       call cmaxMovLen(leng, thick)
 
-      if(leng .lt. IntInfArray(ProcessNo).length) then
+
+      if(leng .lt. IntInfArray(ProcessNo).length ) then
          MoveStat = Truncated
          IntInfArray(ProcessNo).length = leng
          IntInfArray(ProcessNo).thickness  = thick
